@@ -211,8 +211,8 @@ int esd_audio_open()
 #endif
 
     /* set the sound driver audio format for playback */
-    alsa_format.format = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )
-	? /* 16 bit */ SND_PCM_SFMT_S16_LE : /* 8 bit */ SND_PCM_SFMT_U8;
+    alsa_format.format = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )  
+      ? /* 16 bit */ SND_PCM_SFMT_S16 : /* 8 bit */ SND_PCM_SFMT_U8;
     alsa_format.rate = esd_audio_rate;
 
 #ifdef ALSA_5_API
@@ -256,7 +256,7 @@ int esd_audio_open()
     }
 
     if ( params.format.rate != esd_audio_rate || params.format.voices != 2
-                       || params.format.format != SND_PCM_SFMT_S16_LE ) {
+                       || params.format.format != SND_PCM_SFMT_S16 ) {
                       fprintf( stderr, "set format didn't work.");
                       return(-1);
         }
@@ -294,7 +294,7 @@ int esd_audio_open()
 	printf( "error: %s: in snd_pcm_playback_params\n", snd_strerror(ret) );
     }
     if ( alsa_format.rate != esd_audio_rate || alsa_format.channels != 2
-       || alsa_format.format != SND_PCM_SFMT_S16_LE )
+       || alsa_format.format != SND_PCM_SFMT_S16 )
         fprintf( stderr, "set format didn't work.");
 
 #endif /* ALSA_5_API */
@@ -358,6 +358,7 @@ int esd_audio_write( void *buffer, int buf_size )
     }
 
 #ifdef ALSA_5_API
+    status.channel = SND_PCM_CHANNEL_PLAYBACK;
     ret = snd_pcm_channel_status( alsa_sound_handle, &status );
     if( ret ) {
                   if( driver_trace ) fprintf( stderr, "error: %s: in snd_pcm_channel_status\n", snd_strerror(ret) );
