@@ -42,6 +42,7 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <stdio.h>
 
 #ifdef HAVE_MACHINE_SOUNDCARD_H
@@ -63,7 +64,7 @@
 #define REAL_LIBC ((void *) -1L)
 #endif
 
-#if defined(__FreeBSD__) || defined(__bsdi__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__bsdi__)
 typedef unsigned long request_t;
 #else
 typedef int request_t;
@@ -75,6 +76,19 @@ static int settings = 0, done = 0;
 static char *ident = NULL, *mixer = NULL;
 static int use_mixer = 0;
 
+/*
+ * memory mapping emulation
+ */
+static int mmapemu = 0;
+static count_info mmapemu_ocount;
+static char *mmapemu_obuffer = NULL;
+static int mmapemu_osize;
+static long long mmapemu_bytes_per_sec;
+struct timeval mmapemu_last_flush;
+
+#define MMAPEMU_FRAGSTOTAL 16
+#define MMAPEMU_FRAGSIZE ESD_BUF_SIZE
+
 #define OSS_VOLUME_BASE 50
 
 #define ESD_VOL_TO_OSS(left, right) (short int)			\
@@ -114,10 +128,15 @@ dsp_init (void)
 {
   if (!ident)
     {
-      char *str = getenv ("ESDDSP_NAME");
+      const char *str;
+     
+      str = getenv ("ESDDSP_NAME");
       ident = malloc (ESD_NAME_MAX);
       strncpy (ident, (str ? str : "esddsp"), ESD_NAME_MAX);
 
+      str = getenv("ESDDSP_MMAP");
+      mmapemu = str && !strcmp(str,"1");
+
       if (getenv ("ESDDSP_MIXER"))
 	{
 	  use_mixer = 1;
@@ -166,6 +185,34 @@ mix_init (int *esd, int *player)
     }
 }
 
+static void mmapemu_flush(void)
+{
+  struct timeval current_time;
+  long long usec_since_last_flush;
+  long long bytes_to_flush;
+  int frags_to_flush;
+
+  gettimeofday(&current_time, NULL);
+  usec_since_last_flush = 1000000 * (current_time.tv_sec - mmapemu_last_flush.tv_sec) + (current_time.tv_usec - mmapemu_last_flush.tv_usec);
+
+  bytes_to_flush = (mmapemu_bytes_per_sec * usec_since_last_flush) / 1000000;
+  frags_to_flush  = bytes_to_flush / MMAPEMU_FRAGSIZE;
+
+  if (frags_to_flush > MMAPEMU_FRAGSTOTAL)
+    frags_to_flush = MMAPEMU_FRAGSTOTAL;
+
+  if (frags_to_flush > 0)
+    mmapemu_last_flush = current_time;
+
+  while (frags_to_flush-- > 0)
+    {
+      write(sndfd, &mmapemu_obuffer[mmapemu_ocount.ptr], MMAPEMU_FRAGSIZE);
+      mmapemu_ocount.ptr += MMAPEMU_FRAGSIZE;
+      mmapemu_ocount.ptr %= mmapemu_osize;
+      mmapemu_ocount.blocks++;
+      mmapemu_ocount.bytes += MMAPEMU_FRAGSIZE;
+    }
+}
 
 int
 open (const char *pathname, int flags, ...)
@@ -185,7 +232,7 @@ open (const char *pathname, int flags, ...)
 
   if (!strcmp (pathname, "/dev/dsp"))
     {
-      if (!getenv ("ESPEAKER"))
+      if (!getenv ("ESPEAKER") && !mmapemu)
 	{
           int ret;
 
@@ -240,24 +287,68 @@ dspctl (int fd, request_t request, void *argp)
       break;
 
     case SNDCTL_DSP_GETBLKSIZE:
-      *arg = ESD_BUF_SIZE;
+      if (mmapemu)
+      	*arg = MMAPEMU_FRAGSIZE;
+      else
+      	*arg = ESD_BUF_SIZE;
+     
       break;
 
     case SNDCTL_DSP_GETFMTS:
       *arg = 0x38;
       break;
 
+#ifdef SNDCTL_DSP_GETCAPS
     case SNDCTL_DSP_GETCAPS:
-      *arg = 0;
+      if (mmapemu)
+	*arg = DSP_CAP_MMAP | DSP_CAP_TRIGGER | DSP_CAP_REALTIME;
+      else
+	*arg = 0;
       break;
+#endif
 
     case SNDCTL_DSP_GETOSPACE:
+    case SNDCTL_DSP_GETISPACE:
       {
 	audio_buf_info *bufinfo = (audio_buf_info *) argp;
-	bufinfo->bytes = ESD_BUF_SIZE;
+	if (mmapemu)
+	  {
+	    /* Number of fragments allocated for buffering */
+	    bufinfo->fragstotal = MMAPEMU_FRAGSTOTAL;
+	    /* Size of a fragment in bytes (same as SNDCTL_DSP_GETBLKSIZE) */
+	    bufinfo->fragsize = MMAPEMU_FRAGSIZE;
+	    /* Number of bytes that can be read or written immediately without blocking */
+	    bufinfo->bytes = MMAPEMU_FRAGSIZE;
+	    /* Number of full fragments that can be read or written without blocking */
+	    bufinfo->fragments = 1;
+	  }
+	else
+	  {
+	    /* Number of fragments allocated for buffering */
+	    bufinfo->fragstotal = 1;
+	    /* Size of a fragment in bytes (same as SNDCTL_DSP_GETBLKSIZE) */
+	    bufinfo->fragsize = ESD_BUF_SIZE;
+	    /* Number of bytes that can be read or written immediately without blocking */
+	    bufinfo->bytes = ESD_BUF_SIZE;
+	    /* Number of full fragments that can be read or written without blocking */
+	    bufinfo->fragments = 1;
+	  }
       }
       break;
 
+    case SNDCTL_DSP_GETOPTR:
+      if (mmapemu)
+        {
+	  DPRINTF("esddsp: mmapemu getoptr\n");
+	  mmapemu_flush();
+	  *((count_info *)arg) = mmapemu_ocount;
+	  mmapemu_ocount.blocks = 0;
+	}
+      else
+        {
+	  DPRINTF ("esddsp: SNDCTL_DSP_GETOPTR unsupported\n");
+	}
+      break;
 
     default:
       DPRINTF ("unhandled /dev/dsp ioctl (%x - %p)\n", request, argp);
@@ -279,6 +370,13 @@ dspctl (int fd, request_t request, void *argp)
       if (write (sndfd, ident, ESD_NAME_MAX) != ESD_NAME_MAX)
         return -1;
 
+      if (mmapemu)
+	{
+	  mmapemu_ocount.ptr=mmapemu_ocount.blocks=mmapemu_ocount.bytes=0;
+	  mmapemu_bytes_per_sec = speed * ((fmt & ESD_BITS16) ? 2 : 1) * ((fmt & ESD_STEREO) ? 2 : 1);
+	  gettimeofday(&mmapemu_last_flush, NULL);
+	}
+
       fmt = ESD_STREAM | ESD_PLAY | ESD_MONO;
       speed = 0;
 
@@ -414,6 +512,51 @@ close (int fd)
   return (*func) (fd);
 }
 
+void *
+mmap (void  *start,  size_t length, int prot, int flags, int fd, off_t offset)
+{
+  static void *(*func) (void *, size_t, int, int, int, off_t) = NULL;
+  
+  if (!func)
+    func = (void * (*) (void *, size_t, int, int, int, off_t)) dlsym (REAL_LIBC, "mmap");
+
+  if(fd != sndfd || sndfd == -1)
+    return (*func)(start,length,prot,flags,fd,offset);
+  else
+    {
+      DPRINTF ("esddsp: mmap - start = %x, length = %d, prot = %d\n",
+	       start, length, prot);
+      DPRINTF ("      flags = %d, fd = %d, offset = %d\n",flags, fd,offset);
+      if(mmapemu)
+	{
+	  mmapemu_osize = length;
+	  mmapemu_obuffer = malloc(length);
+	  mmapemu_ocount.ptr = mmapemu_ocount.blocks = mmapemu_ocount.bytes = 0;
+	  return mmapemu_obuffer;
+	}
+      else DPRINTF ("esddsp: /dev/dsp mmap (unsupported, try -m option)...\n");
+    }
+  return (void *)-1;
+}
+
+int
+munmap (void *start, size_t length)
+{
+  static int (*func) (void *, size_t) = NULL;
+
+  if (!func)
+    func = (int (*) (void *, size_t)) dlsym (REAL_LIBC, "munmap");
+
+  if(start != mmapemu_obuffer || mmapemu_obuffer == 0)
+    return (*func)(start,length);
+
+  DPRINTF ("esddsp: /dev/dsp munmap...\n");
+  mmapemu_obuffer = NULL;
+  free(start);
+
+  return 0;
+}
+
 #ifdef MULTIPLE_X11AMP
 
 #include <socketbits.h>
@@ -426,7 +569,7 @@ int
 unlink (const char *filename)
 {
   static int (*func) (const char *) = NULL;
-  char *num;
+  const char *num;
 
   if (!func)
     func = (int (*) (const char *)) dlsym (REAL_LIBC, "unlink");
@@ -434,7 +577,7 @@ unlink (const char *filename)
   if (!strcmp (filename, "/tmp/X11Amp_CTRL") && (num = getenv (ENVSET)))
     {
       char buf[PATH_MAX] = "/tmp/X11Amp_CTRL";
-      strcat (buf, num);
+      strncat (buf, num, (PATH_MAX - strlen(buf) - 1));
       return (*func) (buf); 
     }
   else
@@ -446,7 +589,7 @@ typedef int (*sa_func_t) (int, struct sockaddr *, int);
 static int
 sockaddr_mangle (sa_func_t func, int fd, struct sockaddr *addr, int len)
 {
-  char *num;
+  const char *num;
 
   if (!strcmp (((struct sockaddr_un *) addr)->sun_path, "/tmp/X11Amp_CTRL")
       && (num = getenv(ENVSET)))
@@ -456,9 +599,9 @@ sockaddr_mangle (sa_func_t func, int fd, struct sockaddr *addr, int len)
 
       struct sockaddr_un *new_addr = malloc (len);
 
-      strcat (buf, num);
+      strncat (buf, num, (PATH_MAX - strlen(buf) - 1));
       memcpy (new_addr, addr, len);
-      strcpy (new_addr->sun_path, buf);
+      strncpy (new_addr->sun_path, buf, strlen(new_addr->sun_path));
 
       ret = (*func) (fd, (struct sockaddr *) new_addr, len);
 
@@ -499,4 +642,5 @@ nogcc (void)
   ident = NULL;
 }
 
-#endif /* __GNUC__ */
+#endif
+
