@@ -11,7 +11,7 @@ esd_player_t *esd_monitor_list = NULL;
 
 /*******************************************************************/
 /* for debugging purposes, dump the list of the clients and data */
-void dump_players()
+void dump_players(void)
 {
     esd_player_t *player = esd_players_list;
 
@@ -467,7 +467,7 @@ void monitor_write( void *output_buffer, int length ) {
 int recorder_write( void *buffer, int length ) {
 
     esd_player_t *recorder, *remove = NULL;
-    int wrote_len;
+    int wrote_len = 0;
     
     recorder = esd_recorder_list;
 
